@@ -2,7 +2,7 @@
  * Programmer(s): Cody J. Balos @ LLNL
  * -----------------------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2025, Lawrence Livermore National Security,
+ * Copyright (c) 2025-2026, Lawrence Livermore National Security,
  * University of Maryland Baltimore County, and the SUNDIALS contributors.
  * Copyright (c) 2013-2025, Lawrence Livermore National Security
  * and Southern Methodist University.
@@ -20,8 +20,10 @@
 #ifndef _SUNDIALS_MEMORY_HPP
 #define _SUNDIALS_MEMORY_HPP
 
-#include <memory>
+#include <utility>
+
 #include <sundials/sundials_base.hpp>
+#include <sundials/sundials_classview.hpp>
 #include <sundials/sundials_memory.h>
 
 namespace sundials {
@@ -32,10 +34,7 @@ using BaseMemoryHelper = BaseObject<SUNMemoryHelper_, SUNMemoryHelper_Ops_>;
 namespace experimental {
 struct SUNMemoryHelperDeleter
 {
-  void operator()(SUNMemoryHelper helper)
-  {
-    if (helper) { SUNMemoryHelper_Destroy(helper); }
-  }
+  void operator()(SUNMemoryHelper helper) { SUNMemoryHelper_Destroy(helper); }
 };
 
 using SUNMemoryHelperView = ClassView<SUNMemoryHelper, SUNMemoryHelperDeleter>;
