@@ -2,7 +2,7 @@
  * Programmer(s): Daniel R. Reynolds @ UMBC
  *---------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2025, Lawrence Livermore National Security,
+ * Copyright (c) 2025-2026, Lawrence Livermore National Security,
  * University of Maryland Baltimore County, and the SUNDIALS contributors.
  * Copyright (c) 2013-2025, Lawrence Livermore National Security
  * and Southern Methodist University.
@@ -234,7 +234,7 @@ int main(void)
 
   /* Initialize sparse matrix data structure and KLU solver */
   NNZ = 5 * NEQ;
-  A   = SUNSparseMatrix(NEQ, NEQ, NNZ, CSC_MAT, ctx);
+  A   = SUNSparseMatrix(NEQ, NEQ, NNZ, SUN_CSC_MAT, ctx);
   if (check_flag((void*)A, "SUNSparseMatrix", 0)) { return 1; }
 
   LS = SUNLinSol_KLU(y, A, ctx);
@@ -445,7 +445,7 @@ static int Jac(sunrealtype t, N_Vector y, N_Vector fy, SUNMatrix J,
   {
     udata->R = SUNSparseMatrix(SUNSparseMatrix_Rows(J),
                                SUNSparseMatrix_Columns(J),
-                               SUNSparseMatrix_NNZ(J), CSC_MAT, J->sunctx);
+                               SUNSparseMatrix_NNZ(J), SUN_CSC_MAT, J->sunctx);
     if (udata->R == NULL)
     {
       printf("Jacobian calculation error in allocating R matrix!\n");
