@@ -3,7 +3,7 @@
  *      Based on idasRoberts_ASAi_dns.c and modified to use KLU
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2025, Lawrence Livermore National Security,
+ * Copyright (c) 2025-2026, Lawrence Livermore National Security,
  * University of Maryland Baltimore County, and the SUNDIALS contributors.
  * Copyright (c) 2013-2025, Lawrence Livermore National Security
  * and Southern Methodist University.
@@ -229,7 +229,7 @@ int main(int argc, char* argv[])
 
   /* Create sparse SUNMatrix for use in linear solves */
   nnz = NEQ * NEQ;
-  A   = SUNSparseMatrix(NEQ, NEQ, nnz, CSC_MAT, ctx);
+  A   = SUNSparseMatrix(NEQ, NEQ, nnz, SUN_CSC_MAT, ctx);
   if (check_retval((void*)A, "SUNSparseMatrix", 0)) { return (1); }
 
   /* Create KLU SUNLinearSolver object (one thread) */
@@ -373,7 +373,7 @@ int main(int argc, char* argv[])
   if (check_retval(&retval, "IDASetMaxNumStepsB", 1)) { return (1); }
 
   /* Create sparse SUNMatrix for use in linear solves */
-  AB = SUNSparseMatrix(NEQ, NEQ, nnz, CSC_MAT, ctx);
+  AB = SUNSparseMatrix(NEQ, NEQ, nnz, SUN_CSC_MAT, ctx);
   if (check_retval((void*)AB, "SUNSparseMatrix", 0)) { return (1); }
 
   /* Create KLU SUNLinearSolver object (one thread) */
