@@ -3,7 +3,7 @@
  *                Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2025, Lawrence Livermore National Security,
+ * Copyright (c) 2025-2026, Lawrence Livermore National Security,
  * University of Maryland Baltimore County, and the SUNDIALS contributors.
  * Copyright (c) 2013-2025, Lawrence Livermore National Security
  * and Southern Methodist University.
@@ -15,14 +15,13 @@
  * SPDX-License-Identifier: BSD-3-Clause
  * SUNDIALS Copyright End
  * -----------------------------------------------------------------
- * Example problem for KINSOL (parallel machine case) using the BBD
- * preconditioner.
+ * Example (parallel) using the BBD preconditioner:
  *
  * This example solves a nonlinear system that arises from a system
  * of partial differential equations. The PDE system is a food web
- * population model, with predator-prey interaction and diffusion on
- * the unit square in two dimensions. The dependent variable vector
- * is the following:
+ * population model, with predator-prey interaction and diffusion
+ * on the unit square in two dimensions. The dependent variable
+ * vector is the following:
  *
  *       1   2         ns
  * c = (c , c ,  ..., c  )     (denoted by the variable cc)
@@ -91,23 +90,24 @@
  * --------------------------------------------------------------------------
  */
 
-#include <kinsol/kinsol.h>        /* access to KINSOL func., consts.      */
-#include <kinsol/kinsol_bbdpre.h> /* access to BBD preconditioner         */
 #include <math.h>
-#include <mpi.h>
-#include <nvector/nvector_parallel.h> /* access to MPI parallel N_Vector      */
 #include <stdio.h>
 #include <stdlib.h>
-#include <sundials/sundials_dense.h> /* use generic dense solver in precond. */
-#include <sundials/sundials_math.h>  /* access to SUNMAX, SUNRabs, SUNRsqrt  */
-#include <sundials/sundials_types.h> /* defs. of sunrealtype, sunindextype      */
+
+#include <mpi.h>
+
+#include <kinsol/kinsol.h>            /* access to KINSOL func., consts.      */
+#include <kinsol/kinsol_bbdpre.h>     /* access to BBD preconditioner         */
+#include <nvector/nvector_parallel.h> /* access to MPI parallel N_Vector      */
+#include <sundials/sundials_dense.h>  /* use generic dense solver in precond. */
+#include <sundials/sundials_types.h>  /* defs. of sunrealtype, sunindextype   */
 #include <sunlinsol/sunlinsol_spgmr.h> /* access to SPGMR SUNLinearSolver      */
 
 /* Problem Constants */
 
-#define NUM_SPECIES \
-  6 /* must equal 2*(number of prey or predators)
-                              number of prey = number of predators       */
+/* must equal 2*(number of prey or predators)
+   number of prey = number of predators       */
+#define NUM_SPECIES 6
 
 #define NPEX     2              /* number of processors in the x-direction  */
 #define NPEY     2              /* number of processors in the y-direction  */
@@ -136,12 +136,12 @@
 
 /* User-defined vector access macro: IJ_Vptr */
 
-/* IJ_Vptr is defined in order to translate from the underlying 3D structure
-   of the dependent variable vector to the 1D storage scheme for an N-vector.
-   IJ_Vptr(vv,i,j) returns a pointer to the location in vv corresponding to
-   indices is = 0, jx = i, jy = j.    */
+/* IJ_Vptr is defined in order to translate from the underlying 3D structure of
+   the dependent variable vector to the 1D storage scheme for an N_Vector.
+   IJ_Vptr(vptr,i,j) returns a pointer to the location in vptr corresponding to
+   indices is = 0, jx = i, jy = j. */
 
-#define IJ_Vptr(vv, i, j) (&NV_Ith_P(vv, i * NUM_SPECIES + j * NSMXSUB))
+#define IJ_Vptr(vptr, i, j) ((vptr) + (i) * NUM_SPECIES + (j) * NSMXSUB)
 
 /* Type : UserData
    contains problem constants and extended array */
@@ -153,7 +153,7 @@ typedef struct
   sunrealtype *cox, *coy;
   sunrealtype ax, ay, dx, dy;
   sunindextype Nlocal;
-  int mx, my, ns, np;
+  sunindextype mx, my, ns, np;
   sunrealtype cext[NUM_SPECIES * (MXSUB + 2) * (MYSUB + 2)];
   int my_pe, isubx, isuby, nsmxsub, nsmxsub2;
   MPI_Comm comm;
@@ -183,7 +183,7 @@ static void PrintOutput(int my_pe, MPI_Comm comm, N_Vector cc);
 static void PrintFinalStats(void* kmem);
 static void WebRate(sunrealtype xx, sunrealtype yy, sunrealtype* cxy,
                     sunrealtype* ratesxy, void* user_data);
-static sunrealtype DotProd(int size, sunrealtype* x1, sunrealtype* x2);
+static sunrealtype DotProd(sunindextype size, sunrealtype* x1, sunrealtype* x2);
 static void BSend(MPI_Comm comm, int my_pe, int isubx, int isuby, int dsizex,
                   int dsizey, sunrealtype* cdata);
 static void BRecvPost(MPI_Comm comm, MPI_Request request[], int my_pe,
@@ -201,30 +201,21 @@ static int check_retval(void* retvalvalue, const char* funcname, int opt, int id
 
 int main(int argc, char* argv[])
 {
-  SUNContext sunctx;
-  int globalstrategy;
-  sunindextype Nlocal;
-  sunrealtype fnormtol, scsteptol, dq_rel_uu;
-  N_Vector cc, sc, constraints;
-  UserData data;
-  int retval, maxl, maxlrst;
-  int my_pe, npes, npelast = NPEX * NPEY - 1;
-  void* kmem;
-  SUNLinearSolver LS;
-  sunindextype mudq, mldq, mukeep, mlkeep;
-  MPI_Comm comm;
-
-  cc = sc = constraints = NULL;
-  kmem                  = NULL;
-  LS                    = NULL;
-  data                  = NULL;
+  /* Reusable return flag */
+  int retval = 0;
 
   /* Get processor number and total number of pe's */
   MPI_Init(&argc, &argv);
-  comm = MPI_COMM_WORLD;
+  MPI_Comm comm = MPI_COMM_WORLD;
+
+  int npes;
   MPI_Comm_size(comm, &npes);
+
+  int my_pe;
   MPI_Comm_rank(comm, &my_pe);
 
+  int npelast = NPEX * NPEY - 1;
+
   if (npes != NPEX * NPEY)
   {
     if (my_pe == 0)
@@ -237,7 +228,8 @@ int main(int argc, char* argv[])
     return (1);
   }
 
-  /* Create the SUNDIALS simulation context that all SUNDIALS objects require */
+  /* Create the SUNDIALS context that all SUNDIALS objects require */
+  SUNContext sunctx;
   retval = SUNContext_Create(comm, &sunctx);
   if (check_retval(&retval, "SUNContext_Create", 1, my_pe))
   {
@@ -247,10 +239,10 @@ int main(int argc, char* argv[])
   /* Allocate memory, and set problem data, initial values, tolerances */
 
   /* Set local vector length */
-  Nlocal = NUM_SPECIES * MXSUB * MYSUB;
+  sunindextype Nlocal = NUM_SPECIES * MXSUB * MYSUB;
 
   /* Allocate and initialize user data block */
-  data = AllocUserData();
+  UserData data = AllocUserData();
   if (check_retval((void*)data, "AllocUserData", 2, my_pe))
   {
     MPI_Abort(comm, 1);
@@ -258,15 +250,15 @@ int main(int argc, char* argv[])
   InitUserData(my_pe, Nlocal, comm, data);
 
   /* Set global strategy retval */
-  globalstrategy = KIN_NONE;
+  int globalstrategy = KIN_NONE;
 
   /* Allocate and initialize vectors */
-  cc = N_VNew_Parallel(comm, Nlocal, NEQ, sunctx);
+  N_Vector cc = N_VNew_Parallel(comm, Nlocal, NEQ, sunctx);
   if (check_retval((void*)cc, "N_VNew_Parallel", 0, my_pe))
   {
     MPI_Abort(comm, 1);
   }
-  sc = N_VNew_Parallel(comm, Nlocal, NEQ, sunctx);
+  N_Vector sc = N_VNew_Parallel(comm, Nlocal, NEQ, sunctx);
   if (check_retval((void*)sc, "N_VNew_Parallel", 0, my_pe))
   {
     MPI_Abort(comm, 1);
@@ -276,7 +268,7 @@ int main(int argc, char* argv[])
   {
     MPI_Abort(comm, 1);
   }
-  constraints = N_VNew_Parallel(comm, Nlocal, NEQ, sunctx);
+  N_Vector constraints = N_VNew_Parallel(comm, Nlocal, NEQ, sunctx);
   if (check_retval((void*)constraints, "N_VNew_Parallel", 0, my_pe))
   {
     MPI_Abort(comm, 1);
@@ -285,30 +277,33 @@ int main(int argc, char* argv[])
 
   SetInitialProfiles(cc, sc);
 
-  fnormtol  = FTOL;
-  scsteptol = STOL;
+  sunrealtype fnormtol  = FTOL;
+  sunrealtype scsteptol = STOL;
 
-  /* Call KINCreate/KINInit to initialize KINSOL:
-     nvSpec  points to machine environment data
+  /* Call KINCreate/KINInit to initialize KINSOL.
      A pointer to KINSOL problem memory is returned and stored in kmem. */
-  kmem = KINCreate(sunctx);
+  void* kmem = KINCreate(sunctx);
   if (check_retval((void*)kmem, "KINCreate", 0, my_pe)) { MPI_Abort(comm, 1); }
 
   /* Vector cc passed as template vector. */
   retval = KINInit(kmem, func, cc);
   if (check_retval(&retval, "KINInit", 1, my_pe)) { MPI_Abort(comm, 1); }
+
   retval = KINSetUserData(kmem, data);
   if (check_retval(&retval, "KINSetUserData", 1, my_pe)) { MPI_Abort(comm, 1); }
+
   retval = KINSetConstraints(kmem, constraints);
   if (check_retval(&retval, "KINSetConstraints", 1, my_pe))
   {
     MPI_Abort(comm, 1);
   }
+
   retval = KINSetFuncNormTol(kmem, fnormtol);
   if (check_retval(&retval, "KINSetFuncNormTol", 1, my_pe))
   {
     MPI_Abort(comm, 1);
   }
+
   retval = KINSetScaledStepTol(kmem, scsteptol);
   if (check_retval(&retval, "KINSetScaledStepTol", 1, my_pe))
   {
@@ -321,8 +316,9 @@ int main(int argc, char* argv[])
 
   /* Create SUNLinSol_SPGMR object with right preconditioning and the
      maximum Krylov dimension maxl */
-  maxl = 20;
-  LS   = SUNLinSol_SPGMR(cc, SUN_PREC_RIGHT, maxl, sunctx);
+  int maxl = 20;
+
+  SUNLinearSolver LS = SUNLinSol_SPGMR(cc, SUN_PREC_RIGHT, maxl, sunctx);
   if (check_retval((void*)LS, "SUNLinSol_SPGMR", 0, my_pe))
   {
     MPI_Abort(comm, 1);
@@ -336,8 +332,9 @@ int main(int argc, char* argv[])
   }
 
   /* Set the maximum number of restarts */
-  maxlrst = 2;
-  retval  = SUNLinSol_SPGMRSetMaxRestarts(LS, maxlrst);
+  int maxlrst = 2;
+
+  retval = SUNLinSol_SPGMRSetMaxRestarts(LS, maxlrst);
   if (check_retval(&retval, "SUNLinSol_SPGMRSetMaxRestarts", 1, my_pe))
   {
     MPI_Abort(comm, 1);
@@ -347,9 +344,11 @@ int main(int argc, char* argv[])
      band-block-diagonal preconditioner, and specify the local and
      communication functions func_local and gcomm=NULL (all communication
      needed for the func_local is already done in func). */
-  dq_rel_uu = ZERO;
-  mudq = mldq = 2 * NUM_SPECIES - 1;
-  mukeep = mlkeep = NUM_SPECIES;
+  sunrealtype dq_rel_uu = ZERO;
+  sunindextype mudq     = 2 * NUM_SPECIES - 1;
+  sunindextype mldq     = 2 * NUM_SPECIES - 1;
+  sunindextype mukeep   = NUM_SPECIES;
+  sunindextype mlkeep   = NUM_SPECIES;
 
   /* Initialize BBD preconditioner */
   retval = KINBBDPrecInit(kmem, Nlocal, mudq, mldq, mukeep, mlkeep, dq_rel_uu,
@@ -392,13 +391,6 @@ int main(int argc, char* argv[])
   return (0);
 }
 
-/* Readability definitions used in other routines below */
-
-#define acoef (data->acoef)
-#define bcoef (data->bcoef)
-#define cox   (data->cox)
-#define coy   (data->coy)
-
 /*
  *--------------------------------------------------------------------
  * FUNCTIONS CALLED BY KINSOL
@@ -529,6 +521,10 @@ static int func_local(sunindextype Nlocal, N_Vector cc, N_Vector fval,
   dely   = data->dy;
   shifty = (MXSUB + 2) * NUM_SPECIES;
 
+  sunrealtype* ccdata = N_VGetArrayPointer(cc);
+  sunrealtype* rdata  = N_VGetArrayPointer(data->rates);
+  sunrealtype* fdata  = N_VGetArrayPointer(fval);
+
   for (jy = 0; jy < MYSUB; jy++)
   {
     yy = dely * (jy + isuby * MYSUB);
@@ -536,9 +532,9 @@ static int func_local(sunindextype Nlocal, N_Vector cc, N_Vector fval,
     for (jx = 0; jx < MXSUB; jx++)
     {
       xx  = delx * (jx + isubx * MXSUB);
-      cxy = IJ_Vptr(cc, jx, jy);
-      rxy = IJ_Vptr(data->rates, jx, jy);
-      fxy = IJ_Vptr(fval, jx, jy);
+      cxy = IJ_Vptr(ccdata, jx, jy);
+      rxy = IJ_Vptr(rdata, jx, jy);
+      fxy = IJ_Vptr(fdata, jx, jy);
 
       WebRate(xx, yy, cxy, rxy, user_data);
 
@@ -559,8 +555,8 @@ static int func_local(sunindextype Nlocal, N_Vector cc, N_Vector fval,
         dcxri = cext[offsetcr + is] - cext[offsetc + is];
 
         /* compute the value at xx , yy */
-        fxy[is] = (coy)[is] * (dcyui - dcydi) + (cox)[is] * (dcxri - dcxli) +
-                  rxy[is];
+        fxy[is] = (data->coy)[is] * (dcyui - dcydi) +
+                  (data->cox)[is] * (dcxri - dcxli) + rxy[is];
 
       } /* end of is loop */
 
@@ -599,7 +595,7 @@ static int func(N_Vector cc, N_Vector fval, void* user_data)
 static void WebRate(sunrealtype xx, sunrealtype yy, sunrealtype* cxy,
                     sunrealtype* ratesxy, void* user_data)
 {
-  int i;
+  sunindextype i;
   sunrealtype fac;
   UserData data;
 
@@ -607,14 +603,14 @@ static void WebRate(sunrealtype xx, sunrealtype yy, sunrealtype* cxy,
 
   for (i = 0; i < NUM_SPECIES; i++)
   {
-    ratesxy[i] = DotProd(NUM_SPECIES, cxy, acoef[i]);
+    ratesxy[i] = DotProd(NUM_SPECIES, cxy, data->acoef[i]);
   }
 
   fac = ONE + ALPHA * xx * yy;
 
   for (i = 0; i < NUM_SPECIES; i++)
   {
-    ratesxy[i] = cxy[i] * (bcoef[i] * fac + ratesxy[i]);
+    ratesxy[i] = cxy[i] * (data->bcoef[i] * fac + ratesxy[i]);
   }
 }
 
@@ -622,9 +618,9 @@ static void WebRate(sunrealtype xx, sunrealtype yy, sunrealtype* cxy,
  * Dot product routine for sunrealtype arrays
  */
 
-static sunrealtype DotProd(int size, sunrealtype* x1, sunrealtype* x2)
+static sunrealtype DotProd(sunindextype size, sunrealtype* x1, sunrealtype* x2)
 {
-  int i;
+  sunindextype i;
   sunrealtype *xx1, *xx2, temp = ZERO;
 
   xx1 = x1;
@@ -650,10 +646,10 @@ static UserData AllocUserData(void)
 
   data = (UserData)malloc(sizeof *data);
 
-  acoef = SUNDlsMat_newDenseMat(NUM_SPECIES, NUM_SPECIES);
-  bcoef = (sunrealtype*)malloc(NUM_SPECIES * sizeof(sunrealtype));
-  cox   = (sunrealtype*)malloc(NUM_SPECIES * sizeof(sunrealtype));
-  coy   = (sunrealtype*)malloc(NUM_SPECIES * sizeof(sunrealtype));
+  data->acoef = SUNDlsMat_newDenseMat(NUM_SPECIES, NUM_SPECIES);
+  data->bcoef = (sunrealtype*)malloc(NUM_SPECIES * sizeof(sunrealtype));
+  data->cox   = (sunrealtype*)malloc(NUM_SPECIES * sizeof(sunrealtype));
+  data->coy   = (sunrealtype*)malloc(NUM_SPECIES * sizeof(sunrealtype));
 
   return (data);
 }
@@ -692,10 +688,10 @@ static void InitUserData(int my_pe, sunindextype Nlocal, MPI_Comm comm,
 
   for (i = 0; i < np; i++)
   {
-    a1 = &(acoef[i][np]);
-    a2 = &(acoef[i + np][0]);
-    a3 = &(acoef[i][0]);
-    a4 = &(acoef[i + np][np]);
+    a1 = &(data->acoef[i][np]);
+    a2 = &(data->acoef[i + np][0]);
+    a3 = &(data->acoef[i][0]);
+    a4 = &(data->acoef[i + np][np]);
 
     /*  Fill in the portion of acoef in the four quadrants, row by row */
     for (j = 0; j < np; j++)
@@ -707,17 +703,17 @@ static void InitUserData(int my_pe, sunindextype Nlocal, MPI_Comm comm,
     }
 
     /* and then change the diagonal elements of acoef to -AA */
-    acoef[i][i]           = -AA;
-    acoef[i + np][i + np] = -AA;
+    data->acoef[i][i]           = -AA;
+    data->acoef[i + np][i + np] = -AA;
 
-    bcoef[i]      = BB;
-    bcoef[i + np] = -BB;
+    data->bcoef[i]      = BB;
+    data->bcoef[i + np] = -BB;
 
-    cox[i]      = DPREY / dx2;
-    cox[i + np] = DPRED / dx2;
+    data->cox[i]      = DPREY / dx2;
+    data->cox[i + np] = DPRED / dx2;
 
-    coy[i]      = DPREY / dy2;
-    coy[i + np] = DPRED / dy2;
+    data->coy[i]      = DPREY / dy2;
+    data->coy[i + np] = DPRED / dy2;
   }
 }
 
@@ -727,12 +723,11 @@ static void InitUserData(int my_pe, sunindextype Nlocal, MPI_Comm comm,
 
 static void FreeUserData(UserData data)
 {
-  SUNDlsMat_destroyMat(acoef);
-  free(bcoef);
-  free(cox);
-  free(coy);
+  SUNDlsMat_destroyMat(data->acoef);
+  free(data->bcoef);
+  free(data->cox);
+  free(data->coy);
   N_VDestroy(data->rates);
-
   free(data);
 }
 
@@ -746,6 +741,9 @@ static void SetInitialProfiles(N_Vector cc, N_Vector sc)
   sunrealtype *cloc, *sloc;
   sunrealtype ctemp[NUM_SPECIES], stemp[NUM_SPECIES];
 
+  sunrealtype* ccdata = N_VGetArrayPointer(cc);
+  sunrealtype* scdata = N_VGetArrayPointer(sc);
+
   /* Initialize arrays ctemp and stemp used in the loading process */
   for (i = 0; i < NUM_SPECIES / 2; i++)
   {
@@ -763,8 +761,8 @@ static void SetInitialProfiles(N_Vector cc, N_Vector sc)
   {
     for (jx = 0; jx < MXSUB; jx++)
     {
-      cloc = IJ_Vptr(cc, jx, jy);
-      sloc = IJ_Vptr(sc, jx, jy);
+      cloc = IJ_Vptr(ccdata, jx, jy);
+      sloc = IJ_Vptr(scdata, jx, jy);
       for (i = 0; i < NUM_SPECIES; i++)
       {
         cloc[i] = ctemp[i];
@@ -783,7 +781,7 @@ static void PrintHeader(int globalstrategy, int maxl, int maxlrst,
                         sunindextype mukeep, sunindextype mlkeep,
                         sunrealtype fnormtol, sunrealtype scsteptol)
 {
-  printf("\nPredator-prey test problem--  KINSol (parallel-BBD version)\n\n");
+  printf("Predator-prey test problem -- KINSol (parallel-BBD version)\n\n");
   printf("Mesh dimensions = %d X %d\n", MX, MY);
   printf("Number of species = %d\n", NUM_SPECIES);
   printf("Total system size = %d\n\n", NEQ);
@@ -799,9 +797,6 @@ static void PrintHeader(int globalstrategy, int maxl, int maxlrst,
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("Tolerance parameters:  fnormtol = %Lg   scsteptol = %Lg\n", fnormtol,
          scsteptol);
-#elif defined(SUNDIALS_DOUBLE_PRECISION)
-  printf("Tolerance parameters:  fnormtol = %g   scsteptol = %g\n", fnormtol,
-         scsteptol);
 #else
   printf("Tolerance parameters:  fnormtol = %g   scsteptol = %g\n", fnormtol,
          scsteptol);
@@ -811,9 +806,6 @@ static void PrintHeader(int globalstrategy, int maxl, int maxlrst,
 #if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("At all mesh points:  %Lg %Lg %Lg   %Lg %Lg %Lg\n", PREYIN, PREYIN,
          PREYIN, PREDIN, PREDIN, PREDIN);
-#elif defined(SUNDIALS_DOUBLE_PRECISION)
-  printf("At all mesh points:  %g %g %g   %g %g %g\n", PREYIN, PREYIN, PREYIN,
-         PREDIN, PREDIN, PREDIN);
 #else
   printf("At all mesh points:  %g %g %g   %g %g %g\n", PREYIN, PREYIN, PREYIN,
          PREDIN, PREDIN, PREDIN);
@@ -821,7 +813,7 @@ static void PrintHeader(int globalstrategy, int maxl, int maxlrst,
 }
 
 /*
- * Print sample of current cc values
+ * Print sampled values of current cc
  */
 
 static void PrintOutput(int my_pe, MPI_Comm comm, N_Vector cc)
@@ -864,8 +856,6 @@ static void PrintOutput(int my_pe, MPI_Comm comm, N_Vector cc)
       if ((is % 6) * 6 == is) { printf("\n"); }
 #if defined(SUNDIALS_EXTENDED_PRECISION)
       printf(" %Lg", ct[is]);
-#elif defined(SUNDIALS_DOUBLE_PRECISION)
-      printf(" %g", ct[is]);
 #else
       printf(" %g", ct[is]);
 #endif
@@ -877,8 +867,6 @@ static void PrintOutput(int my_pe, MPI_Comm comm, N_Vector cc)
       if ((is % 6) * 6 == is) { printf("\n"); }
 #if defined(SUNDIALS_EXTENDED_PRECISION)
       printf(" %Lg", tempc[is]);
-#elif defined(SUNDIALS_DOUBLE_PRECISION)
-      printf(" %g", tempc[is]);
 #else
       printf(" %g", tempc[is]);
 #endif
@@ -911,7 +899,7 @@ static void PrintFinalStats(void* kmem)
   retval = KINGetNumLinFuncEvals(kmem, &nfeSG);
   check_retval(&retval, "KINGetNumLinFuncEvals", 1, 0);
 
-  printf("Final Statistics.. \n");
+  printf("Final Statistics..\n");
   printf("nni    = %5ld    nli   = %5ld\n", nni, nli);
   printf("nfe    = %5ld    nfeSG = %5ld\n", nfe, nfeSG);
   printf("nps    = %5ld    npe   = %5ld     ncfl  = %5ld\n", nps, npe, ncfl);
